#download gsea2-2.09.jar from broad institute and place it in th esame folder as this script

rm(list=ls())

learnNOW=FALSE; #if set to TRUE then the homology mapping between rat and human is learned now and can take long time (~12h)
nproc=32; #number of processors to use (assuming a linux like system with 32 or more processors)

gs=scan("c2.cp.v3.1.symbols.gmt",sep="\n",what="character")
template=read.table("template_prediction_SC3_gene_sets_human.txt",sep="\t",header=TRUE,row.names=1)
#read gene expression
rA=read.table("GEx_rat_train.txt",sep="\t",header=TRUE,row.names=1)
rB=read.table("GEx_rat_test.txt",sep="\t",header=TRUE,row.names=1)
hA=read.table("GEx_human_train.txt",sep="\t",header=TRUE,row.names=1)

#required libraries
source("oraF2.R")
source("gseapF.r")
library(limma)
library(parallel)




f2=function(x){al=unlist(strsplit(x,split="\t"));al[-(1:2)]}
f1=function(x){al=unlist(strsplit(x,split="\t"));al[1]}
kr=sapply(gs,f2)
names(kr)<-sapply(gs,f1)
template2=template
kr=kr[rownames(template)]


NI=2000


#function to transform FDR q-values into confidence level
mytr=0.25
mytr5=0.45

tans=function(fdrP){
fdrPconf=rep(NA,length(fdrP))
fdrPconf[fdrP<mytr]<-1
fdrPconf[fdrP>mytr5]<-0
zxz=fdrP[fdrP>=mytr & fdrP<=mytr5]
aa=-1/(mytr5-mytr)
bb=1-aa*mytr
fdrPconf[fdrP>=mytr & fdrP<=mytr5]<- aa*zxz+bb
fdrPconf
}

#################
#process gene expression to find pairs


 unig=unlist(kr)
 names(unig)<-NULL
unig=unique(unig)



#keep in the gene sets only genes that are on the human array
mygslist=kr
 for(i in names(mygslist)){
  mygslist[[i]]=intersect(mygslist[[i]],rownames(hA))
 }


#get ranks in rat
 stims=setdiff(unique(unlist(lapply(strsplit(colnames(rA),split="_"),"[[",1))),"DME")
getdataaslist=function(set){
i=set
datt=rA[,grep(paste(i,"_",sep=""),colnames(rA))]
colnames(datt)
ba=unlist(lapply(strsplit(colnames(datt),split="_"),"[[",3))[1]
datt=cbind(rA[,grep(paste(i,"_",sep=""),colnames(rA))],
rA[,intersect(grep(ba,colnames(rA)),grep("DME",colnames(rA)))])

 list = list(dataset=i, disease=i, dat.m=datt, 
 ano=data.frame(ID=names(datt),Group=ifelse(substr(names(datt),1,3)=="DME","c","d")),
  design="Not paired", annotation=NULL,      
            targetGeneSets="i")                                                  
        names(list) = c("dataset", "disease", "dat.m", "ano",                
            "design", "annotation", "targetGeneSets")
return(list)
}        

rgenes=rownames(rA)
rnkRat=NULL
for(sti in stims){
rnkd=oraF2(sti,mygslist,minsize)
or=match(rgenes,rnkd$ID)
rnkRat=cbind(rnkRat,or/dim(rnkd)[1])
}
colnames(rnkRat)<-stims
rownames(rnkRat)<-rgenes


##############
#get ranks in human
 stims=setdiff(unique(unlist(lapply(strsplit(colnames(hA),split="_"),"[[",1))),"DME")
getdataaslist=function(set){
i=set
datt=hA[,grep(paste(i,"_",sep=""),colnames(hA))]
colnames(datt)
ba=unlist(lapply(strsplit(colnames(datt),split="_"),"[[",3))[1]
datt=cbind(hA[,grep(paste(i,"_",sep=""),colnames(hA))],
hA[,intersect(grep(ba,colnames(hA)),grep("DME",colnames(hA)))])

 list = list(dataset=i, disease=i, dat.m=datt, 
 ano=data.frame(ID=names(datt),Group=ifelse(substr(names(datt),1,3)=="DME","c","d")),
  design="Not paired", annotation=NULL,      
            targetGeneSets="i")                                                  
        names(list) = c("dataset", "disease", "dat.m", "ano",                
            "design", "annotation", "targetGeneSets")
return(list)
}        


hgenes=rownames(hA)
rnkH=NULL
for(sti in stims){
rnkd=oraF2(sti,mygslist,minsize)
or=match(hgenes,rnkd$ID)
rnkH=cbind(rnkH,or/dim(rnkd)[1])
}
colnames(rnkH)<-stims
rownames(rnkH)<-hgenes



set.seed(1)
prediall=rownames(rnkRat)




respli=list()
for(i in 1:length(hgenes)){
respli[[i]]<-hgenes[i]
}


#define a function that finds for each rat gene a human gene
parf=function(x,predi){
cors=NULL
for(p in predi){
 we=abs(rnkH[x,]-0.5)
 we=we^2/sum(we^2)
 mc=sum(abs(rnkRat[p,]-rnkH[x,])*we)
 cors=c(cors,mc)
 }
 ord=order(cors,decreasing=FALSE)[1]
 fe=predi[ord]
fe
}


if(learnNOW){
if(require(parallel)){  
respf=mclapply(respli,parf,predi=prediall,mc.cores=nproc)
}else{
 respf=lapply(respli,parf,predi=prediall) 
}

respf=unlist(respf)
names(respf)<-hgenes
save(respf,file="respf.RData")
}else{
 load("respf.RData")
}


#handle cases when same rat gene is picked for different human genes
#by transforming the gene list 
myff=function(x){
zz=table(x)
ou=names(zz)[zz==1]
if(sum(zz>1)>=1){ou2=zz[zz>1]; 
for(mi in 1:length(ou2)){
zx=paste(names(ou2)[mi],1:(ou2[mi]-1),sep="___")
ou=c(ou,names(ou2)[mi],zx)}
}
ou
}

 for(i in names(mygslist)){
  mygslist[[i]]=myff(respf[mygslist[[i]]])
 }


 #and by transforming the expression matrix
alcr=unique(unlist(mygslist))
alcr=alcr[grep("___",alcr)]
names(alcr)<-NULL
alcrn=unlist(lapply(strsplit(alcr,split="___"),"[[",1))
tm=rA; tm=tm[alcrn,];rownames(tm)<-alcr;rA=rbind(rA,tm)
tm=rB; tm=tm[alcrn,];rownames(tm)<-alcr;rB=rbind(rB,tm)


 

#run gene set analysis on the test set to get predictions

stims=colnames(template)

getdataaslist=function(set){
i=set
datt=rB[,grep(paste(i,"_",sep=""),colnames(rB))]
colnames(datt)
ba=unlist(lapply(strsplit(colnames(datt),split="_"),"[[",3))[1]
datt=cbind(rB[,grep(paste(i,"_",sep=""),colnames(rB))],
rA[,intersect(grep(ba,colnames(rA)),grep("DME",colnames(rA)))])

 list = list(dataset=i, disease=i, dat.m=datt, 
 ano=data.frame(ID=names(datt),Group=ifelse(substr(names(datt),1,3)=="DME","c","d")),
  design="Not paired", annotation=NULL,      
            targetGeneSets="i")                                                  
        names(list) = c("dataset", "disease", "dat.m", "ano",                
            "design", "annotation", "targetGeneSets")
return(list)
}        


#make predictions by running GSEA
for(sti in stims){
system("rm -f ./tmpgsea/*.*")
system("rm -f ./tmpgseap/*.*")
system("rm -Rf ./tmpgseap/*")

res=gseapF(set=sti,mygslist,minsize=3)
template2[as.character(res$NAME),sti]=tans(res$FDR.q.val)

}

template[is.na(template)]<-0
template2[is.na(template2)]<-0

write.table(template2,sep="\t",file="PRB_SC3_predictions.txt")

